@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -83,7 +55,6 @@ import org.apache.http.impl.client.BasicCookieStore;
 import org.apache.http.impl.client.CloseableHttpClient;
 import org.apache.http.impl.client.HttpClients;
 import org.apache.http.ssl.SSLContextBuilder;
-import org.apache.http.util.TextUtils;
 
 @UtilityClass
 public class RequestUtil {
@@ -122,7 +93,7 @@ public class RequestUtil {
 
     private static final int USER_AGENT_MAXLEN = 100;
 
-    private static final Pattern USER_AGENT_PATTERN = Pattern.compile("^(.*)\\.(.*)\\.Mail/(\\S*) \\(.*\\)$");
+    private static final Pattern USER_AGENT_PATTERN = Pattern.compile("^(.*)\\.(.*)\\.Mail/(\\S*) \\(OS:(.*); device:(.*)\\)$");
 
     private static final String HEADER_CACHE_CONTROL = "Cache-control";
 
@@ -174,16 +145,18 @@ public class RequestUtil {
             throw ApiFacadeException.badRequest(ApiFacadeException.Code.MISSING_COOKIES, "Cookies missing");
         }
         String hostname = getHostHeader(request);
+        String xHostname = getXHostHeader(request);
         String userAgent = getUserAgent(request);
         String trackingId = getTrackingId(request);
         String contextId = getContextId(request);
         String userId = getUserId(request);
-        HostConfiguration hostConfiguration = ConfigurationHelper.getConfiguration(request);
-        checkUserAgent(userAgent, hostConfiguration);
-        return enhanceSessionData(request, hostConfiguration, hostname, userAgent, trackingId, contextId, userId, new SessionData(session, cookies));
+        UserAgentInfo userAgentInfo = parseUserAgent(userAgent);
+        HostConfiguration hostConfiguration = ConfigurationHelper.getConfiguration(hostname, xHostname, userAgentInfo);
+        checkUserAgent(userAgentInfo, hostConfiguration);
+        return enhanceSessionData(request, hostConfiguration, hostname, xHostname, userAgent, trackingId, contextId, userId, new SessionData(session, cookies));
     }
 
-    public static SessionData enhanceSessionData(HttpServletRequest request, HostConfiguration configuration, String hostname, String userAgent, String trackingId, String contextId, String userId, SessionData sessionData) {
+    public static SessionData enhanceSessionData(HttpServletRequest request, HostConfiguration configuration, String hostname, String xHostname, String userAgent, String trackingId, String contextId, String userId, SessionData sessionData) {
         if (trackingId != null) {
             sessionData.setProperty(SessionData.PROPERTY_TRACKING_ID, trackingId);
         }
@@ -199,6 +172,7 @@ public class RequestUtil {
         }
         return sessionData
                 .withHost(hostname)
+                .withXHost(xHostname)
                 .withForwardedFor(getForwardedForWithRemoteIp(request))
                 .withForwardedProto(getForwardedProto(request))
                 .withUserAgent(userAgent)
@@ -213,11 +187,7 @@ public class RequestUtil {
         return userAgent;
     }
 
-    public void checkUserAgent(String userAgent, HostConfiguration hostConfiguration) {
-        if (Strings.isEmpty(userAgent)) {
-            throw ApiFacadeException.missingHeader(HEADER_USER_AGENT);
-        }
-        UserAgentInfo userAgentInfo = parseUserAgent(userAgent);
+    public void checkUserAgent(UserAgentInfo userAgentInfo, HostConfiguration hostConfiguration) {
         if (!hostConfiguration.isAllowedBrand(userAgentInfo.getBrand())) {
             throw ApiFacadeException.forbidden();
         }
@@ -237,7 +207,7 @@ public class RequestUtil {
         }
         Semver clientVersion = new Semver(clientVersionStr, SemverType.LOOSE);
         Semver minimumClientVersion = new Semver(minimumClientVersionStr, SemverType.LOOSE);
-        return !clientVersion.isLowerThan(minimumClientVersion);
+        return minimumClientVersion.isLowerThanOrEqualTo(clientVersion);
     }
 
     private String getMinimumClientVersion(Map<String, String> minimumClientVersions, UserAgentInfo userAgentInfo) {
@@ -306,20 +276,25 @@ public class RequestUtil {
     }
 
     public UserAgentInfo parseUserAgent(String userAgent) {
-        if (TextUtils.isEmpty(userAgent) || userAgent.length() > USER_AGENT_MAXLEN) {
+        if (Strings.isEmpty(userAgent)) {
+            throw ApiFacadeException.missingHeader(HEADER_USER_AGENT);
+        }
+        if (userAgent.length() > USER_AGENT_MAXLEN) {
             throw ApiFacadeException.invalidUserAgent();
         }
         Matcher matcher = USER_AGENT_PATTERN.matcher(userAgent);
         if (!matcher.find()) {
             throw ApiFacadeException.invalidUserAgent();
         }
-        if (matcher.groupCount() != 3) {
+        if (matcher.groupCount() != 5) {
             throw ApiFacadeException.invalidUserAgent();
         }
         String brand = matcher.group(1);
         String platform = matcher.group(2);
         String clientVersion = matcher.group(3);
-        return new UserAgentInfo(brand, platform, clientVersion);
+        String osVersion = matcher.group(4).trim();
+        String device = matcher.group(5).trim();
+        return new UserAgentInfo(brand, platform, clientVersion, osVersion, device);
     }
 
     /*private*/ String getIfNoneMatch(HttpServletRequest request) {
@@ -331,11 +306,11 @@ public class RequestUtil {
     }
 
     public String getHostHeader(HttpServletRequest request) {
-        String value = request.getHeader(HEADER_X_HOST);
-        if (value == null) {
-            value = request.getHeader(HEADER_HOST);
-        }
-        return value;
+        return request.getHeader(HEADER_HOST);
+    }
+
+    public String getXHostHeader(HttpServletRequest request) {
+        return request.getHeader(HEADER_X_HOST);
     }
 
     public String getLanguage(HttpServletRequest request) {
